/**The MIT License (MIT)
 
 Copyright (c) 2023 by ArtSuzhou LLC
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 */

// This unit checks for a signal from MASTER every X seconds - when no signal D2(GPIO4)=LOW, when signal D2(GPIO4)=HIGH

#include<ESP8266WiFi.h>
#include<espnow.h>

#define MY_NAME         "SLAVE_NODE"
#define MY_ROLE         ESP_NOW_ROLE_CONTROLLER         // set the role of this device: CONTROLLER, SLAVE, COMBO
#define RECEIVER_ROLE   ESP_NOW_ROLE_SLAVE              // set the role of the receiver
#define WIFI_CHANNEL    4                               // Channel used in the connection

#define LED_PIN D4 // 14 Pin number for data transmitted LED
#define RELAY_PIN D2 // Pin number for relay pin

uint8_t receiverAddress[] = {0xB4, 0xE6, 0x2D, 0x23, 0x0F, 0x29};     // please update this with the MAC address of the receiver

int data = 0;

void setup() {
  Serial.begin(115200);     // initialize serial port

  Serial.print("Initializing...");
  Serial.println(MY_NAME);
  Serial.print("My MAC address is: ");
  Serial.println(WiFi.macAddress());

  WiFi.mode(WIFI_STA);
  WiFi.disconnect();        // we do not want to connect to a WiFi network

  pinMode(LED_PIN, OUTPUT);
  pinMode(RELAY_PIN, OUTPUT);

  if (esp_now_init() != 0) {
    Serial.println("ESP-NOW initialization failed");
    return;
  } else {
    Serial.println("Initialized.");
  }

  esp_now_set_self_role(MY_ROLE);
  esp_now_register_send_cb(transmissionComplete);   // this function will get called once all data is sent
  esp_now_add_peer(receiverAddress, RECEIVER_ROLE, WIFI_CHANNEL, NULL, 0);
}

/** callback when data is sent from Master to Slave **/
void transmissionComplete(uint8_t *mac_addr, uint8_t transmissionStatus) {
  Serial.print("I sent my data -> "); 
  Serial.println(data);
  char macStr[18];
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print("Last Packet Sent to: "); Serial.println(macStr);
  Serial.print("Last Packet Send Status: "); Serial.println(transmissionStatus == 0 ? "Delivery Success" : "Delivery Fail");
  if (transmissionStatus == 0) {
    digitalWrite(LED_PIN, HIGH);
    digitalWrite(RELAY_PIN, HIGH);
  } else {
    digitalWrite(LED_PIN, LOW);
    digitalWrite(RELAY_PIN, LOW);
  }
  
}

void loop() {
  data++;
  esp_now_send(receiverAddress, (uint8_t *) &data, sizeof(data));  
  Serial.println(digitalRead(RELAY_PIN) == 1 ? "Relay On" : "Relay OFF");  

  if (digitalRead(RELAY_PIN) == 1){
    delay(2000);
  }else{
    delay(200);  
  }    // increase the dealy if to reduce the intermittent power cut due to signal loss
}
